/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.client;

import java.util.logging.Level;

import org.keycloak.admin.client.CreatedResponseUtil;
import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.resource.ClientResource;
import org.keycloak.admin.client.resource.RealmResource;
import org.keycloak.representations.idm.ClientRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.keycloak.KeycloakResultParser;
import lombok.extern.java.Log;

@Component
@Log
class KeycloakClientRemoteService {

	@Autowired
	private Keycloak keycloak;

	public void updateClient(ClientRepresentation client, String realm) {
		getClientResource(realm, client.getId()).update(client);
	}

	public String createClient(ClientRepresentation client, String realm) {
		log.log(Level.FINE, "Creating client {0} in realm {1}", new String[] { client.getId(), realm });
		var response = getRealm(realm).clients().create(client);
		KeycloakResultParser.parseCreatedResponse(response);
		return CreatedResponseUtil.getCreatedId(response);
	}

	public void updateClientRole(RoleRepresentation role, String clientId, String realm) {
		getClientResource(realm, clientId).roles().get(role.getName()).update(role);
	}

	public void addClientRole(RoleRepresentation role, String clientId, String realm) {
		getClientResource(realm, clientId).roles().create(role);
	}

	private ClientResource getClientResource(String realm, String clientId) {
		return getRealm(realm).clients().get(clientId);
	}

	private RealmResource getRealm(String realm) {
		return keycloak.realm(realm);
	}
}
