/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.client;

import java.util.List;

import org.keycloak.representations.idm.ClientRepresentation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.operator.keycloak.KeycloakGenericRemoteService;

@Component
class KeycloakClientService {

	@Autowired
	private KeycloakClientRemoteService remoteService;

	@Autowired
	private KeycloakGenericRemoteService genericRemoteService;

	@Autowired
	private ProtocolMapperRepresentationHelper mapperRepresentationBuilder;

	@Autowired
	private KeycloakClientMapper mapper;

	void createOrUpdateClient(OzgKeycloakClientSpec spec, String namespace) {
		genericRemoteService.getByClientId(spec.getClientId(), namespace)
				.ifPresentOrElse(existingClient -> updateClient(existingClient, spec, namespace),
						() -> createClient(spec, namespace));
	}

	void updateClient(ClientRepresentation existingClient, OzgKeycloakClientSpec spec, String realm) {
		var clientRepresentation = mapper.update(existingClient, spec);
		remoteService.updateClient(clientRepresentation, realm);

		addOrUpdateClientRoles(spec, realm, existingClient.getId());
	}

	void createClient(OzgKeycloakClientSpec spec, String realm) {
		var clientRepresentation = mapper.map(spec);
		setProtocolMapper(clientRepresentation);
		var realClientId = remoteService.createClient(clientRepresentation, realm);

		addOrUpdateClientRoles(spec, realm, realClientId);
	}

	void setProtocolMapper(ClientRepresentation clientRepresentation) {
		clientRepresentation.setProtocolMappers(List.of(
				mapperRepresentationBuilder.createOrganisationsEinheitIdMapper(),
				mapperRepresentationBuilder.createOzgCloudUserIdMapper(),
				mapperRepresentationBuilder.createOrganisationeEinheitIdLdapMapper(),
				mapperRepresentationBuilder.createClientRolesMapper()));
	}

	void addOrUpdateClientRoles(OzgKeycloakClientSpec spec, String realm, String realClientId) {
		spec.getClientRoles().forEach(
				roleSpec -> genericRemoteService.getClientRole(roleSpec.getName(), realClientId, realm)
						.ifPresentOrElse(
								existingRole -> remoteService.updateClientRole(mapper.updateRole(existingRole, roleSpec), realClientId, realm),
								() -> remoteService.addClientRole(mapper.mapRole(roleSpec), realClientId, realm)));
	}
}
