/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import java.util.Optional;
import java.util.logging.Level;

import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import lombok.extern.java.Log;

@Log
@Component
class KeycloakUserService {

	@Autowired
	private KeycloakUserRemoteService remoteService;

	@Autowired
	private UserSecretService userSecretService;

	@Autowired
	private KeycloakUserMapper userMapper;

	public void createOrUpdateUser(OzgKeycloakUserSpec userSpec, String namespace) {
		if (!userSecretService.exists(userSpec, namespace)) {
			log.log(Level.INFO, "Update password...");
			var userPassword = userSpec.getKeycloakUser().getPassword();
			var password = StringUtils.isEmpty(userPassword) ? generatePassword() : userPassword;
			userSpec.getKeycloakUser().setPassword(password);

			log.log(Level.INFO, "Create secret for user: " + userSpec.getKeycloakUser().getUsername());
			userSecretService.create(userSpec, namespace);
		}

		remoteService.getUserByName(userSpec.getKeycloakUser().getUsername(), namespace)
				.ifPresentOrElse(existingUser -> remoteService.updateUser(userMapper.update(existingUser, userSpec), namespace),
						() -> remoteService.createUser(userMapper.map(userSpec), namespace));
	}

	String generatePassword() {
		var upperCaseCharacter = RandomStringUtils.randomAlphabetic(1).toUpperCase();
		var randomString = RandomStringUtils.randomAlphanumeric(7);
		return upperCaseCharacter + randomString;
	}

	public void deleteUser(OzgKeycloakUserSpec userSpec, String namespace) {
		Optional.of(userSpec)
				.map(userMapper::map)
				.flatMap(keycloakUser -> remoteService.getUserByName(keycloakUser.getUsername(), namespace))
				.ifPresent(keycloakUser -> remoteService.deleteUser(keycloakUser.getId(), namespace));
	}
}
