/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.user;

import java.util.ArrayList;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonSetter;
import com.fasterxml.jackson.annotation.Nulls;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

@Getter
@Setter
@Builder
@NoArgsConstructor
@AllArgsConstructor
@JsonIgnoreProperties(ignoreUnknown = true)
class OzgKeycloakUserSpec {

	@JsonProperty("keep_after_delete")
	private boolean keepAfterDelete;

	@JsonProperty("keycloak_user")
	private KeycloakUserSpecUser keycloakUser;

	@Getter
	@Setter
	@Builder
	@NoArgsConstructor
	@AllArgsConstructor
	static class KeycloakUserSpecUser {

		@JsonProperty("username")
		private String username;

		@JsonProperty("email")
		private String email;

		@JsonProperty("first_name")
		private String firstName;

		@JsonProperty("last_name")
		private String lastName;

		@JsonProperty("password")
		private String password;

		@JsonProperty("groups")
		@Builder.Default
		private List<KeycloakUserSpecUserGroup> groups = new ArrayList<>();

		@JsonProperty("client_roles")
		@Builder.Default
		private List<KeycloakUserSpecClientRole> clientRoles = new ArrayList<>();

		@JsonSetter(nulls = Nulls.AS_EMPTY)
		public void setGroups(List<KeycloakUserSpecUserGroup> groups) {
			this.groups = groups;
		}

		@JsonSetter(nulls = Nulls.AS_EMPTY)
		public void setClientRoles(List<KeycloakUserSpecClientRole> clientRoles) {
			this.clientRoles = clientRoles;
		}
	}

	@Getter
	@Setter
	@Builder
	@NoArgsConstructor
	@AllArgsConstructor
	static class KeycloakUserSpecUserGroup {

		@JsonProperty("name")
		private String name;
	}

	@Getter
	@Setter
	@Builder
	@NoArgsConstructor
	@AllArgsConstructor
	static class KeycloakUserSpecClientRole {

		@JsonProperty("name")
		private String clientId;

		@JsonProperty("role")
		private String roleName;
	}
}
