package de.ozgcloud.operator.keycloak.user;

import java.util.Objects;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import io.fabric8.kubernetes.api.model.Secret;
import io.fabric8.kubernetes.client.dsl.Resource;
import io.fabric8.kubernetes.client.extension.ResourceAdapter;

@Component
class UserSecretService {

	@Autowired
	private UserNameConverter userNameConverter;
	@Autowired
	private UserSecretBuilder secretBuilder;
	@Autowired
	private KubernetesRemoteService kubernetesRemoteService;

	public boolean exists(OzgKeycloakUserSpec userSpec, String namespace) {
		return Objects.nonNull(getUserSecret(userSpec, namespace).get());
	}

	public void create(OzgKeycloakUserSpec userSpec, String namespace) {
		var secretName = userNameConverter.toSecretName(userSpec.getKeycloakUser());

		var credentialsSecret = secretBuilder.build(secretName, userSpec.getKeycloakUser(), namespace);

		var adapter = createResourceAdpater(getUserSecret(userSpec, namespace));
		adapter.create(credentialsSecret);
	}

	ResourceAdapter<Secret> createResourceAdpater(Resource<Secret> secretResource) {
		return new ResourceAdapter<>(secretResource);
	}

	Resource<Secret> getUserSecret(OzgKeycloakUserSpec userSpec, String namespace) {
		var secretName = userNameConverter.toSecretName(userSpec.getKeycloakUser());

		return kubernetesRemoteService.getSecret(namespace, secretName);
	}
}
