/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.operator.keycloak.realm;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.operator.keycloak.OzgCustomResourceStatus;

class KeycloakRealmReconcilerTest {

	@Spy
	@InjectMocks
	private KeycloakRealmReconciler reconciler;

	@Mock
	private KeycloakRealmService service;

	@Nested
	class TestReconcile {

		@Test
		void shouldCallServiceAddRealm() {
			OzgKeycloakRealm realm = OzgKeycloakRealmTestFactory.create();

			reconciler.reconcile(realm, null);

			verify(service).createRealm(realm.getSpec(), OzgKeycloakRealmTestFactory.METADATA_NAMESPACE);
		}

		@Test
		void shouldReturnUpdateStatus() {
			var response = reconciler.reconcile(OzgKeycloakRealmTestFactory.create(), null);

			assertThat(response.getResource()).isNotNull();
		}

		@Test
		void shouldSetStatusOk() {
			var response = reconciler.reconcile(OzgKeycloakRealmTestFactory.create(), null);

			assertThat(response.getResource().getStatus().getStatus()).isEqualTo(OzgCustomResourceStatus.OK);
		}
	}

	@Nested
	class TestDelete {

		@Test
		void shouldCallServiceDelete() {
			OzgKeycloakRealm realm = OzgKeycloakRealmTestFactory.create();

			reconciler.cleanup(realm, null);

			verify(service).deleteRealm(OzgKeycloakRealmTestFactory.METADATA_NAMESPACE);
		}
	}
}
